"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.JsonTemplateEngine = void 0;
const constants_1 = require("./constants");
const lexer_1 = require("./lexer");
const parser_1 = require("./parser");
const translator_1 = require("./translator");
const utils_1 = require("./utils");
class JsonTemplateEngine {
    constructor(fn) {
        this.fn = fn;
    }
    static create(templateOrExpr, compileTimeBindings) {
        return new JsonTemplateEngine(this.compile(templateOrExpr, compileTimeBindings));
    }
    static createSync(templateOrExpr, compileTimeBindings) {
        return new JsonTemplateEngine(this.compileSync(templateOrExpr, compileTimeBindings));
    }
    static compileSync(templateOrExpr, compileTimeBindings) {
        return Function(constants_1.DATA_PARAM_KEY, constants_1.BINDINGS_PARAM_KEY, this.translate(templateOrExpr, compileTimeBindings));
    }
    static compile(templateOrExpr, compileTimeBindings) {
        return utils_1.CommonUtils.CreateAsyncFunction(constants_1.DATA_PARAM_KEY, constants_1.BINDINGS_PARAM_KEY, this.translate(templateOrExpr, compileTimeBindings));
    }
    static parse(template) {
        const lexer = new lexer_1.JsonTemplateLexer(template);
        const parser = new parser_1.JsonTemplateParser(lexer);
        return parser.parse();
    }
    static translate(templateOrExpr, compileTimeBindings) {
        if (typeof templateOrExpr === 'string') {
            return this.translateTemplate(templateOrExpr, compileTimeBindings);
        }
        return this.translateExpression(templateOrExpr, compileTimeBindings);
    }
    static translateTemplate(template, compileTimeBindings) {
        return this.translateExpression(this.parse(template), compileTimeBindings);
    }
    static translateExpression(expr, compileTimeBindings) {
        const translator = new translator_1.JsonTemplateTranslator(expr, compileTimeBindings);
        return translator.translate();
    }
    evaluate(data, bindings = {}) {
        return this.fn(data || {}, bindings);
    }
}
exports.JsonTemplateEngine = JsonTemplateEngine;
